@@ -2,9 +2,8 @@
 
 #
 # Fsdb.pm
-# $Id: 535f2d6db51a9c848cff016eb9af024dc4522614 $
 #
-# Copyright (C) 1991-2015 by John Heidemann <johnh@isi.edu>
+# Copyright (C) 1991-2016 by John Heidemann <johnh@isi.edu>
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License,
@@ -34,7 +33,7 @@ Fsdb - a flat-text database for shell scripting
 
 
 =cut
-our $VERSION = '2.58';
+our $VERSION = '2.62';
 
 =head1 SYNOPSIS
 
@@ -96,7 +95,7 @@ in the first, second, and third columns.
 Rather than hand-code scripts to do each special case, Fsdb provides
 higher-level functions.  Although it's often easy throw together a
 custom script to do any single task, I believe that there are several
-advantages to using this library:
+advantages to using Fsdb:
 
 =over 4
 
@@ -220,8 +219,9 @@ also at the web page L<http://www.rdb.com/>).  Fsdb is an incompatible
 re-implementation of their idea without any accelerated indexing or
 forms support.  (But it's free, and probably has better statistics!).
 
-Fsdb-2.x supports threading and will exploit multiple processors or cores,
+Fsdb-2.x will exploit multiple processors or cores,
 and provides Perl-level support for input, output, and threaded-pipelines.
+(As of Fsdb-2.44 it no longer uses Perl threading, just processes.)
 
 Installation instructions follow at the end of this document. 
 Fsdb-2.x requires Perl 5.8 to run.  
@@ -234,18 +234,36 @@ L<http://www.isi.edu/~johnh/SOFTWARE/FSDB/index.html>.
 
 =head1 WHAT'S NEW
 
-=head2 2.58, 2015-04-30
-Bugfix in L<dbmerge>
+=head2 2.62, 2016-11-29
+A new L<yaml_to_db> and other minor improvements.
 
 =over 4
 
+=item ENHANCEMENT
+
+Documentation for L<xml_to_db> now includes sample output.
+
+=item NEW
+
+L<yaml_to_db> converts a specific form of YAML to fsdb.
+
 =item BUG FIX
 
-Fixed a case where L<dbmerge> suffered mojobake in endgame mode.
-This bug surfaced when L<dbsort> was applied to large files
-(big enough to require merging) with unicode in them;
-the symptom was soemthing like:
-  Wide character in print at /usr/lib64/perl5/IO/Handle.pm line 420, <GEN12> line 111.
+The test suite now uses C<diff -c -b> rather than C<diff -cb>
+to make OpenBSD-5.9 happier, I hope.
+
+=item ENHANCEMENT
+
+Comments that log operations at the end of each file now do simple
+quoting of spaces.  (It is not guaranteed to be fully shell-compliant.)
+
+=item ENHANCEMENT
+
+There is a new standard option, C<--header>,
+allowing one to specify an Fsdb header for inputs that lack it.
+Currently it is supported by L<dbcoldefine>,
+L<dbrowuniq>, L<dbmapreduce>, L<dbmultistats>, L<dbsort>,
+L<dbpipeline>.
 
 =back
 
@@ -305,7 +323,7 @@ Or, if you want to install it somewhere else, change the first line to
 and it will go in your home directory's F<bin>, etc.
 (See L<ExtUtil::MakeMaker(3)> for more details.)
 
-Fsdb requires perl 5.8 or later and uses ithreads.
+Fsdb requires perl 5.8 or later.
 
 A test-suite is available, run it with
 
@@ -761,6 +779,12 @@ Input data from file I.
 
 Write data out to file O.
 
+=item B<--header> H
+
+Use H as the full Fsdb header, rather than reading a header from
+then input.  This option is particularly useful when using Fsdb
+under Hadoop, where split files don't have heades.
+
 =item B<--nolog>.
 
 Skip logging the program in a trailing comment.
@@ -1053,23 +1077,7 @@ that is a bug that should be documented
 on the tool's manual page or ideally fixed.
 
 Fsdb does depend on Perl's correctness, and Perl (and Fsdb) have
-some bugs.  Fsdb should work on perl from version 5.10 onward,
-but its use of threads gives bogus warnings in some versions of perl:
-
-=over 4
-
-=item *
-perl-5.10 and 5.12 generate warnings "unbalanced string table refcount" and "scalars leaked"
-in L<dbmapreduce>
-
-=item *
-perl-5.10 generates warning "Attempt to free unreferenced scalar"
-in L<dbmultistats>.
-
-=back
-
-To my knowledge these do not the correctness of the output,
-other than cluttering it up with warnings.
+some bugs.  Fsdb should work on perl from version 5.10 onward.
 
 
 =head1 HISTORY
@@ -1131,7 +1139,10 @@ should be able to read and write files in any format.
 
 Fsdb-2.0 preserves backwards compatibility where possible,
 but breaks it where necessary to accomplish the above goals.
-In August 2008, fsdb-2.7 was declared preferred over the 1.x versions.
+In August 2008, Fsdb-2.7 was declared preferred over the 1.x versions.
+Benchmarking in 2013 showed that threading performed much worse than
+just using pipes, so Fsdb-2.44 uses threading "style",
+but implemented with processes (via my "Freds" library).
 
 =head2 Contributors
 
@@ -1160,7 +1171,8 @@ Xue Cai,
 Michael McQuaid,
 Christopher Meng,
 Calvin Ardi,
-H. Merijn Brand.
+H. Merijn Brand,
+Lan Wei.
 
 Fsdb includes datasets contributed from NIST (F<DATA/nist_zarr13.fsdb>),
 from
@@ -3167,7 +3179,6 @@ Test suites changes to be robust to exact line numbers of failures,
 since different Perl releases fail on different lines.
 L<https://bugzilla.redhat.com/show_bug.cgi?id=1158380>
 
-
 =back
 
 
@@ -3224,7 +3235,7 @@ Now fixed.
 =back
 
 =head2 2.55, 2015-01-05
-many spelling fixes and dbcolmovingstats tests are more robust to different numeric precision
+many spelling fixes and L<dbcolmovingstats> tests are more robust to different numeric precision
 
 =over 4
 
@@ -3244,7 +3255,7 @@ and thanks to H. Merijn Brand for the bug report.
 =item BUG FIX
 
 Many, many spelling bugs found by 
-H. Merijn Brand for the bug report.
+H. Merijn Brand; thanks for the bug report.
 
 =item INCOMPATBLE CHANGE
 
@@ -3270,6 +3281,107 @@ Added missing BuildRequires for C<XML::Simple>.
 
 =back
 
+=head2 2.57, 2015-04-29
+Minor changes, with better performance from L<dbmulitstats>.
+
+=over 4
+
+=item BUG FIX
+
+L<dbfilecat> now honors C<--remove-inputs> (previously it didn't).
+This omission meant that L<dbmapreduce> (and L<dbmultistats>) would accumulate
+files in F</tmp> when running.  Bad news for inputs with 4M keys.
+
+=item ENHANCMENT
+
+L<dbmultistats> should be faster with lots of small keys.
+L<dbcolstats> now supports C<-k> to get some of the functionality of
+L<dbmultistats> (if data is pre-sorted and median/quartiles are not required).
+
+L<dbfilecat> now honors C<--remove-inputs> (previously it didn't).
+This omission meant that L<dbmapreduce> (and L<dbmultistats>) would accumulate
+files in F</tmp> when running.  Bad news for inputs with 4M keys.
+
+=back
+
+
+=head2 2.58, 2015-04-30
+Bugfix in L<dbmerge>
+
+=over 4
+
+=item BUG FIX
+
+Fixed a case where L<dbmerge> suffered mojobake in endgame mode.
+This bug surfaced when L<dbsort> was applied to large files
+(big enough to require merging) with unicode in them;
+the symptom was soemthing like:
+  Wide character in print at /usr/lib64/perl5/IO/Handle.pm line 420, <GEN12> line 111.
+
+=back
+
+
+=head2 2.59, 2016-09-01
+Collect a few small bug fixes and documentation improvements.
+
+=over 4
+
+=item BUG FIX
+
+More IO is explicitly marked UTF-8 to avoid Perl's tendency to
+mojibake on otherwise valid unicode input.
+This change helps L<html_table_to_db>.
+
+=item ENHANCEMENT
+
+L<dbcolscorrelate> now crossreferences L<dbcolsregression>.
+
+=item ENHANCEMENT
+
+Documentation for L<dbrowdiff> now clarifies that the default is baseline mode.
+
+=item BUG FIX
+
+L<dbjoin> now propagates C<-T> into the sorting process (if it is required).
+Thanks to Lan Wei for reporting this bug.
+
+=back
+
+
+=head2 2.60, 2016-09-04
+Adds support for hash joins.
+
+=over 4
+
+=item ENHANCEMENT
+
+L<dbjoin> now supports hash joins
+with C<-t lefthash> and C<-t righthash>.
+Hash joins cache a table in memory, but do not require
+that the other table be sorted.
+They are ideal when joining a large table against a small one.
+
+=back
+
+=head2 2.61, 2016-09-05
+Support left and right outer joins.
+
+=over 4
+
+=item ENHANCEMENT
+
+L<dbjoin> now handles left and right outer joins
+with C<-t left> and C<-t right>.
+
+=item ENHANCEMENT
+
+L<dbjoin> hash joins are now selected 
+with C<-m lefthash> and C<-m righthash>
+(not the shortlived C<-t righthash> option).
+(Technically this change is incompatible with Fsdd-2.60, but
+no one but me ever used that version.)
+
+=back
 
 =head1 AUTHOR
 
@@ -3281,7 +3393,7 @@ bug reports and fixes.
 
 =head1 COPYRIGHT
 
-Fsdb is Copyright (C) 1991-2013 by John Heidemann <johnh@isi.edu>.
+Fsdb is Copyright (C) 1991-2016 by John Heidemann <johnh@isi.edu>.
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of version 2 of the GNU General Public License as
