@@ -2,7 +2,7 @@
 
 #
 # Filter.pm
-# $Id$
+# $Id: 7843a6bb9a62b736e670fcf61fdb3e66994cf79f $
 #
 # Copyright (C) 2007-2008 by John Heidemann <johnh@isi.edu>
 #
@@ -499,15 +499,15 @@ sub finish_io_option ($$@) {
 
 =head2 direction_to_stdio
 
-    direction_to_stdio($direction)
+    $fh = direction_to_stdio($direction)
 
 Private internal routing.  Give a filehandle for STDIN or
 STDOUT based on $DIRECTION == 'input or 'output'
 
 =cut
 
-sub direction_to_stdio { # private
-    my($direction) = @_;
+sub direction_to_stdio($;$){ # private
+    my($direction, $encoding) = @_;
     my $fh = new IO::Handle;
     if ($direction eq 'input') {
 	$fh = $fh->fdopen(fileno(STDIN), "r");
@@ -516,6 +516,9 @@ sub direction_to_stdio { # private
     } else {
 	croak "bad direction";
     };
+    $encoding = ":utf8" if (!defined($encoding));
+    binmode $fh, $encoding;
+    return $fh;
 }
 
 
@@ -539,9 +542,8 @@ No return value.
 
 =cut
 
-sub finish_fh_io_option ($$) {
-    my $self = shift @_;
-    my $direction = shift @_;
+sub finish_fh_io_option ($$;$) {
+    my($self, $direction, $encoding) = @_;
     croak "finish_fh_io_option: bad direction $direction\n"
 	if (!($direction eq 'input' || $direction eq 'output'));
     my $token = $self->{"_$direction"};
@@ -557,13 +559,15 @@ sub finish_fh_io_option ($$) {
     } elsif (ref($token) eq '') {
 	# (ref($token) == '' if $token == *main::STDIN or $token eq '-'
 	if ($token eq '-') {
-	    $fh = direction_to_stdio($direction);
+	    $fh = direction_to_stdio($direction, $encoding);
 	} else {
 	    # assume it's a glob to a filehandle
 	    $fh = $token;
 	};
     } else {
 	$fh = new IO::File($token, ($direction eq 'input' ? 'r' : 'w'));
+	$encoding = ":utf8" if (!defined($encoding));
+	binmode $fh, $encoding;
     };
 
     $self->{($direction eq 'input' ? '_in' : '_out')} = $fh;
@@ -617,6 +621,10 @@ sub compute_program_log($) {
        foreach (@{$self->{_orig_argv}}) {
 	    if (ref($_) eq 'CODE') {
 		$args .= " [ANONYMOUS-CODE]";
+	    } elsif ($_ =~ /\s/) {
+	        # should use String::ShellQuote, but don't want the dpeendency
+		$_ =~ s/\'/'\\\''/g;
+		$args .= " '" . $_ . "'";
 	    } else {
 		$args .= " " . $_;
 	    };
