@@ -2,8 +2,8 @@
 
 #
 # dbjoin.pm
-# Copyright (C) 1991-2008 by John Heidemann <johnh@isi.edu>
-# $Id$
+# Copyright (C) 1991-2016 by John Heidemann <johnh@isi.edu>
+# $Id: 653e1186837d37a1668d02763a023b730484a03c $
 #
 # This program is distributed under terms of the GNU general
 # public license, version 2.  See the file COPYING
@@ -34,9 +34,11 @@ with C<-t outer> it will do a natural, full outer join.
 inner joints output records only when there are matches in both tables
 and will omit records that do not match.
 Outer joins output all records from both tables, 
-filling with the empty value as needed.)
+filling with the empty value as needed.
+Right (left) outer joins keep all elements of the right (left)
+table, even those that don't match in the other table.)
 
-By default, data will be sorted lexically,
+By default for non-hash joins, data will be sorted lexically,
 but the usual sorting options can be mixed with the column
 specification.
 
@@ -44,10 +46,17 @@ Because two tables are required,
 input is typically in files.
 Standard input is accessible by the file "-".
 
-=head1 RESOURCE REQUIREMENTS
+=head1 RESOURCE REQUIREMENTS AND PERFORMANCE
 
+Joins can be expensive. 
+Most databases have a query optimizer that 
+knows something about the data and so can select
+algorithms for efficent operation,
+in Fsdb, I<you> are that optimizer.
+
+For I<non-hash joins>:
 If data is already sorted, dbjoin will run more efficiently
-with the C<-S> option.
+by telling dbjoin the data is sorted with the C<-S>.
 
 The resource requirements L<dbjoin> vary.
 If input data is sorted and C<-S> is given,
@@ -75,6 +84,13 @@ if there are many people per country (as one would expect).
 If warning "lots of matching rows accumulating in memory" appears,
 this is the cause and try swapping join order.
 
+For I<hash joins>
+(that is, with C<-m righthash> or C<-m lefthash>):
+all of the right table (the second input) or the left (the first)
+is loaded into memory (and "hashed").
+The other table need not be sorted.
+Runtime is O(n), but memory is O(size of hashed table).
+
 
 =head1 OPTIONS
 
@@ -90,8 +106,14 @@ Default is an I<inner join>.
 =item B<-t TYPE> or B<--type TYPE>
 
 Explicitly specify the join type.
-TYPE must be inner, outer, left, or right.
-Currently only inner and outer are implemented.
+TYPE must be inner, outer, left (outer), right (outer).
+Default: inner.
+
+=item B<-m METHOD> or B<--method METHOD>
+
+Select join method (algorithm).
+Choices are merge, righthash, and lefthash.
+Default: merge.
 
 =item B<-S> or B<--pre-sorted>
 
@@ -264,7 +286,8 @@ sub set_defaults ($) {
     $self->{_pre_sorted} = 0;
     $self->{_sort_argv} = [];
     $self->{_join_type} = 'inner';
-    $self->{_tmpdir} = defined($ENV{'TMPDIR'}) ? $ENV{'TMPDIR'} : "/tmp";
+    $self->{_join_method} = 'merge';
+    $self->set_default_tmpdir;
 }
 
 =head2 parse_options
@@ -290,6 +313,7 @@ sub parse_options ($@) {
 	'e|empty=s' => \$self->{_empty},
 	'i|input=s@' => sub { $self->parse_io_option('inputs', @_); },
 	'log!' => \$self->{_logprog},
+	'm|method=s' => \$self->{_join_method},
 	'o|output=s' => sub { $self->parse_io_option('output', @_); },
 	'S|pre-sorted+' => \$self->{_pre_sorted},
 	't|type=s' => \$self->{_join_type},
@@ -313,22 +337,35 @@ Internal: setup, parse headers.
 
 =cut
 
-sub setup ($) {
+sub setup($) {
     my($self) = @_;
 
     croak $self->{_prog} . ": no sorting key specified.\n"
 	if ($#{$self->{_sort_argv}} == -1);
-    croak $self->{_prog} . ": join types left and right are not currently supported.\n"
-	if ($self->{_join_type} eq 'left' || $self->{_join_type} eq 'right');
     croak $self->{_prog} . ": unknown join type " . $self->{_join_type} . ".\n"
-	if (!($self->{_join_type} eq 'inner' || $self->{_join_type} eq 'outer'));
+	if (!($self->{_join_type} eq 'inner' || $self->{_join_type} eq 'outer' || $self->{_join_type} eq 'left' || $self->{_join_type} eq 'right'));
+    croak $self->{_prog} . ": unknown join method " . $self->{_join_method} . ".\n"
+	if (!($self->{_join_method} eq 'merge' || $self->{_join_method} eq 'lefthash' || $self->{_join_method} eq 'righthash'));
 
     $self->setup_exactly_two_inputs;
 
+    # hash join?
+    my($hashside, $fullside) = (undef, undef);
+    if ($self->{_join_method} eq 'righthash') {
+	($hashside, $fullside) = (1, 0);
+    } elsif ($self->{_join_method} eq 'lefthash') {
+	($hashside, $fullside) = (0, 1);
+    };
+
+    #
     # automatic input sorting?
-    if (!$self->{_pre_sorted}) {
+    #
+    if ($self->{_join_method} eq 'merge' && !$self->{_pre_sorted}) {
+	my(@final_sort_argv) = @{$self->{_sort_argv}};
+	unshift(@final_sort_argv, '-T', $self->{_tmpdir})
+	    if (defined($self->{_tmpdir}));
 	foreach (0..1) {
-	    my($new_reader, $new_fred) = dbpipeline_filter($self->{_inputs}[$_], [-comment_handler => $self->create_delay_comments_sub], dbsort('--nolog', @{$self->{_sort_argv}}));
+	    my($new_reader, $new_fred) = dbpipeline_filter($self->{_inputs}[$_], [-comment_handler => $self->create_delay_comments_sub], dbsort('--nolog', @final_sort_argv));
 	    $self->{_pre_sorted_inputs}[$_] = $self->{_inputs}[$_];
 	    $self->{_ins}[$_] = $new_reader;
 	    $self->{_in_freds}[$_] = $new_fred;
@@ -337,26 +374,52 @@ sub setup ($) {
 	$self->finish_io_option('inputs', -comment_handler => $self->create_delay_comments_sub);
     };
 
+    # can't move next check earlier; it must be fater "finish_io_option".
     croak $self->{_prog} . ": cannot handle input data with different field separators.\n"
 	if (!defined($self->{_ins}[0]->compare($self->{_ins}[1])));
 
     #
     # figure the joined columns
     #
-    my $i = 0;
+    my $i = 0;  # counts output columns
     my %join_keys;
     my $key;
     my @output_columns;
     my %all_keys;
     my @copy_codes = ('', '');
+    my $build_key_from_hashside_code = '';
+    my $build_key_from_fullside_code = '';
+    my $build_hashside_result_code = '';
+    my $build_output_from_hit_code = '';
+    my $merge_output_from_hit_code = '';
+    my $build_output_from_full_code = '';
+    my @hash_input_col_to_hit_col;   # not actullay used
+    my $hit_ncol = 0;
     for $key (@{$self->{_sort_argv}}) {
         next if ($key =~ /^-/);   # we deal with this later
+
+	# figure out details for both sides
 	my @col_i;
 	foreach (0..1) {
 	    $col_i[$_] = $self->{_ins}[$_]->col_to_i($key);
 	    die($self->{_prog} . ": column ``$key'' is not in " . ($_ == 0 ? "left" : "right") ." join source.\n") if (!defined($col_i[$_]));
-	    $copy_codes[$_] .= '$out_fref->[' . $i . '] = $fref[' . $_ . "]->[$col_i[$_]];\n";
+	    # for merge join, details are copy
+	    $copy_codes[$_] .= '$out_fref->[' . $i . '] = $frefs[' . $_ . "]->[$col_i[$_]];\n";
+	    # for hash join, also build the hash key
+	    if ($self->{_join_method} ne 'merge') {
+		if ($_ == $hashside) {
+		    $build_key_from_hashside_code .= '$key .= ' . ($i == 0 ? '' : '$fs . ') . '$fref->[' . $col_i[$_] . ']; ';
+		    $build_hashside_result_code .= ($i == 0 ? '' : ", ") . '$fref->[' . $col_i[$_] . ']';
+		    $hash_input_col_to_hit_col[$col_i[$_]] = $hit_ncol++;
+		    $build_output_from_hit_code .= '$out_fref->[' . $i . '] = $hit->[' . $i . "];\n";
+		};
+		if ($_ == $fullside) {
+		    $build_key_from_fullside_code .= '$key .= ' . ($i == 0 ? '' : '$fs . ') . '$fref->[' . $col_i[$_] . ']; ';
+		    $build_output_from_full_code .= '$out_fref->[' . $i . '] = $fref->[' . $col_i[$_] . "];\n";
+		};
+	    };
 	};
+
 	push(@output_columns, $key);
         $join_keys{$key} = $i;
         $all_keys{$key} = $i;
@@ -373,8 +436,20 @@ sub setup ($) {
 	push(@output_columns, $key);
 	$col_i = $self->{_ins}[0]->col_to_i($key);
 	defined($col_i) or die "assert";
-	$copy_codes[0] .= '$out_fref->[' . $i . '] = $fref[0]->[' . $col_i . '];' . "\n";
+	$copy_codes[0] .= '$out_fref->[' . $i . '] = $frefs[0]->[' . $col_i . '];' . "\n";
 	$all_keys{$key} = $i;
+	if ($self->{_join_method} ne 'merge') {
+	    if (0 == $hashside) {
+		$build_hashside_result_code .= ', $fref->[' . $col_i . ']';
+		my $hit_col = $hit_ncol++;
+		$hash_input_col_to_hit_col[$col_i] = $hit_col;
+		$build_output_from_hit_code .= '$out_fref->[' . $i . '] = $hit->[' . $hit_col . '];';
+		$merge_output_from_hit_code .= '$out_fref->[' . $i . '] = $hit->[' . $hit_col . '];';
+	    };
+	    if (0 == $fullside) {
+		$build_output_from_full_code .= '$out_fref->[' . $i . '] = $fref->[' . $col_i . '];';
+	    };
+	};
         $i++;
     };
     foreach $key (@{$self->{_ins}[1]->cols}) {
@@ -388,46 +463,65 @@ sub setup ($) {
 	push(@output_columns, $key);
 	$col_i = $self->{_ins}[1]->col_to_i($key);
 	defined($col_i) or die "assert";
-	$copy_codes[1] .= '$out_fref->[' . $i . '] = $fref[1]->[' . $col_i . '];' . "\n";
+	$copy_codes[1] .= '$out_fref->[' . $i . '] = $frefs[1]->[' . $col_i . '];' . "\n";
 	$all_keys{$key} = $i;
+	if ($self->{_join_method} ne 'merge') {
+	    if (1 == $hashside) {
+		$build_hashside_result_code .= ', $fref->[' . $col_i . ']';
+		my $hit_col = $hit_ncol++;
+		$hash_input_col_to_hit_col[$col_i] = $hit_col;
+		$build_output_from_hit_code .= '$out_fref->[' . $i . '] = $hit->[' . $hit_col . '];';
+		$merge_output_from_hit_code .= '$out_fref->[' . $i . '] = $hit->[' . $hit_col . '];';
+	    };
+	    if (1 == $fullside) {
+		$build_output_from_full_code .= '$out_fref->[' . $i . '] = $fref->[' . $col_i . '];';
+	    };
+	};
         $i++;
     };
     $self->{_copy_codes} = \@copy_codes;
 
+    $self->{_build_key_from_hashside_code} = $build_key_from_hashside_code;
+    $self->{_build_key_from_fullside_code} = $build_key_from_fullside_code;
+    $self->{_build_hashside_result_code} = $build_hashside_result_code;
+    $self->{_build_output_from_hit_code} = $build_output_from_hit_code;
+    $self->{_merge_output_from_hit_code} = $merge_output_from_hit_code;
+    $self->{_build_output_from_full_code} = $build_output_from_full_code;
+
     $self->finish_io_option('output', -clone => $self->{_ins}[0], -cols => \@output_columns);
 
     #
     # comparision code
     #
-    $self->{_compare_code} = $self->create_compare_code(@{$self->{_ins}}, 'fref[0]', 'fref[1]');;
+    $self->{_compare_code} = $self->create_compare_code(@{$self->{_ins}}, 'frefs[0]', 'frefs[1]');;
     croak $self->{_prog} . ": no join field specified.\n"
 	if (!defined($self->{_compare_code}));
 
     print "COMPARE CODE:\n\t" . $self->{_compare_code} . "\n" if ($self->{_debug});
     foreach (0..1) {
-       $self->{_compare_code_ins}[$_] = $self->create_compare_code($self->{_ins}[$_], $self->{_ins}[$_], "prev_fref[$_]", "fref[$_]");
+       $self->{_compare_code_ins}[$_] = $self->create_compare_code($self->{_ins}[$_], $self->{_ins}[$_], "prev_frefs[$_]", "frefs[$_]");
        croak $self->{_prog} . ": no join field specified.\n"
 	    if (!defined($self->{_compare_code_ins}[$_]));
     };
 }
 
 
-=head2 run
+=head2 run_merge_join
 
-    $filter->run();
+    $filter->run_merge_join();
 
 Internal: run over each rows.
 
 =cut
-sub run ($) {
+sub run_merge_join($) {
     my($self) = @_;
 
     # 
     # Eval compare_sub in this lexical context
     # of our variables.
     #
-    my @prev_fref;
-    my @fref;
+    my @prev_frefs;
+    my @frefs;
     my(@right_frefs);
     my $compare_sub;
     my @check_compare_subs;
@@ -468,18 +562,22 @@ sub run ($) {
     };
     if ($self->{_join_type} eq 'inner') {
 	$emit_non_match_left = $emit_non_match_right = sub {};
+    } elsif ($self->{_join_type} eq 'left') {
+	$emit_non_match_right = sub {};
+    } elsif ($self->{_join_type} eq 'right') {
+	$emit_non_match_left = sub {};
     };
     my $advance_left = sub {
-	$prev_fref[0] = $fref[0];
-	$fref[0] = &{$fastpath_subs[0]}();
-	if (defined($fref[0])) {
+	$prev_frefs[0] = $frefs[0];
+	$frefs[0] = &{$fastpath_subs[0]}();
+	if (defined($frefs[0])) {
 	    &{$check_compare_subs[0]}() <= 0 or die "dbjoin: left stream is unsorted.\n";
 	};
     };
     my $advance_right = sub {
-	$prev_fref[1] = $fref[1];
-	$fref[1] = &{$fastpath_subs[1]}();
-	if (defined($fref[1])) {
+	$prev_frefs[1] = $frefs[1];
+	$frefs[1] = &{$fastpath_subs[1]}();
+	if (defined($frefs[1])) {
 	    &{$check_compare_subs[1]}() <= 0 or die "dbjoin: right stream is unsorted.\n";
 	};
     };
@@ -514,7 +612,7 @@ sub run ($) {
 
     # prime the pump
     foreach (0..1) {
-	$fref[$_] = &{$fastpath_subs[$_]}();
+	$frefs[$_] = &{$fastpath_subs[$_]}();
     };
 
     #
@@ -523,15 +621,15 @@ sub run ($) {
 until_eof:
     for (;;) {
 	# eof on either stream? quit main loop
-	last if (!defined($fref[0]) || !defined($fref[1]));
+	last if (!defined($frefs[0]) || !defined($frefs[1]));
 
 	# eat data until we have a match
 	my $left_right_cmp;
 
 until_match:
 	for (;;) {
-	    defined($fref[0]) or die "assert";
-	    defined($fref[1]) or die "assert";
+	    defined($frefs[0]) or die "assert";
+	    defined($frefs[1]) or die "assert";
 
 	    $left_right_cmp = &{$compare_sub}();
 
@@ -539,12 +637,12 @@ until_match:
 		# left wins, so eat left
 		&{$emit_non_match_left}();
 		&{$advance_left}();
-		last until_eof if (!defined($fref[0]));
+		last until_eof if (!defined($frefs[0]));
 	    } elsif ($left_right_cmp > 0) {
 		# right wins, eat right
 		&{$emit_non_match_right}();
 		&{$advance_right}();
-		last until_eof if (!defined($fref[1]));
+		last until_eof if (!defined($frefs[1]));
 	    } else {
 		last until_match;
 	    };
@@ -554,8 +652,8 @@ until_match:
 	# match, whoo hoo!
 	#
 	$left_right_cmp == 0 or die "assert";
-	defined($fref[0]) or die "assert";
-	defined($fref[1]) or die "assert";
+	defined($frefs[0]) or die "assert";
+	defined($frefs[1]) or die "assert";
 
 	# accumulate rights
 	# Sigh, we save them in memory.
@@ -563,23 +661,23 @@ until_match:
 	@right_frefs = ();
 accumulate_rights:
 	for (;;) {
-	    push(@right_frefs, $fref[1]);
+	    push(@right_frefs, $frefs[1]);
 	    warn "internal warning: dbjoin: lots of matching rows accumulating in memory. Fixes: dbjoin code can spill to disk (not implemented) or dbjoin user can perhaps swap file orders.\n"
 		if ($#right_frefs == 2000);   # just emit warning once
 	    &{$advance_right}();
-	    last accumulate_rights if (!defined($fref[1]));
+	    last accumulate_rights if (!defined($frefs[1]));
 	    $left_right_cmp = &{$compare_sub}();
 	    last accumulate_rights if ($left_right_cmp != 0);
 	};
-	(!defined($fref[1]) || $left_right_cmp != 0) or die "assert";
+	(!defined($frefs[1]) || $left_right_cmp != 0) or die "assert";
 	#
 	# Ok, this is a bit gross, but we do it anyway.
 	# Right is now one beyond a match.
 	# Save it aside and we'll come back to it later.
-	# This way we can iterate with $fref[1] over our saved @right_frefs,
+	# This way we can iterate with $frefs[1] over our saved @right_frefs,
 	# keeping our &{$fns}()'s happy.
 	#
-	my $right_fref_past_match = $fref[1];
+	my $right_fref_past_match = $frefs[1];
 
 	# now walk lefts
 
@@ -588,32 +686,32 @@ walk_lefts:
 	for (;;) {
 	    &{$copy_left_to_out_fref}();
 	    foreach my $matching_right (@right_frefs) {
-		$fref[1] = $matching_right;  # hacky, but satisifys next line's call
+		$frefs[1] = $matching_right;  # hacky, but satisifys next line's call
 	    	&{$copy_right_to_out_fref}();
 		&{$out_fastpath_sub}($out_fref);
 	    };
 	    &{$advance_left}();
-	    last walk_lefts if (!defined($fref[0]));
+	    last walk_lefts if (!defined($frefs[0]));
 	    $left_right_cmp = &{$compare_sub}();
 	    last walk_lefts if ($left_right_cmp != 0);
 	};
-	(!defined($fref[0]) || $left_right_cmp != 0) or die "assert";
+	(!defined($frefs[0]) || $left_right_cmp != 0) or die "assert";
 	# Put back our one-beyond right.  Could even be eof.
-	$fref[1] = $right_fref_past_match; 
+	$frefs[1] = $right_fref_past_match; 
 
 	# ok, we're now past a match,
 	# and maybe at eof on one stream.
 	# loop back to try again.
-	(!defined($fref[0]) || !defined($fref[1]) || $left_right_cmp != 0) or die "assert";
+	(!defined($frefs[0]) || !defined($frefs[1]) || $left_right_cmp != 0) or die "assert";
     };
 
     # Ok, now at least one side or the other is eof.
     # so drain both sides.
-    while (defined($fref[0])) {
+    while (defined($frefs[0])) {
 	&{$emit_non_match_left}();
 	&{$advance_left}();
     };
-    while (defined($fref[1])) {
+    while (defined($frefs[1])) {
 	&{$emit_non_match_right}();
 	&{$advance_right}();
     };
@@ -628,9 +726,178 @@ walk_lefts:
 }
 
 
+=head2 run_hash_join
+
+    $filter->run_hash_join();
+
+Internal: run over each rows, doing a hash join.
+
+=cut
+sub run_hash_join($$$) {
+    my($self, $hashside, $fullside) = @_;
+
+    #
+    # A basic hash join:  (This is Wikipedia's "Classic hash join".)
+    #
+    # Build phase:
+    #
+    # Load the hashed table into memory,
+    # building a hash table %h.
+    #
+    # Probe phase:
+    #
+    # Then walk the larger table,
+    # checking against the hash table for matches.
+    #
+    # We currently require it fit in memory.
+    #
+
+    #
+    # Build.
+    #
+    my @fastpath_subs;
+    foreach (0..1) {
+	$fastpath_subs[$_] = $self->{_ins}[$_]->fastpath_sub();
+    };
+    my $out_fastpath_sub = $self->{_out}->fastpath_sub();
+
+    my %hashed_table;
+    my %hashed_table_overflow;
+    my $fref;
+    my $overflow_count = 0;
+
+    my $build_key_from_hashside;
+    my $build_key_from_fullside;
+    my $fs = $self->{_out}->fs;
+    my $code = '$build_key_from_hashside = sub {' . "\n\t" . 'my($key) = ""; ' . $self->{_build_key_from_hashside_code} .
+		"\n\t" . 'my @a = ( ' . $self->{_build_hashside_result_code} . ");\n\t" . 'return($key,\@a);' . "\n};\n" .
+		'$build_key_from_fullside = sub {' . "\n\t" . 'my($key) = ""; ' . $self->{_build_key_from_fullside_code} .
+		"\n\t" . 'return $key;' . "\n};\n";
+    eval $code;
+    $@ && croak $self->{_prog} . ":  internal eval error in hash build code: $@.\n$code\n";
+
+    while ($fref = &{$fastpath_subs[$hashside]}()) {
+	# build the hash entry
+	my($key, $aref) = &{$build_key_from_hashside}($fref);
+	if (defined($hashed_table{$key})) {
+	    # overflow
+	    if (ref($hashed_table{$key}) eq 'SCALAR') {
+		# already overflowed
+		push(@{$hashed_table_overflow{$key}}, $aref);
+	    } else {
+		# new overflow
+		die "internal error: confused about overflow on $key\n"
+		    if (defined($hashed_table_overflow{$key}));
+		my @new_overflow = ($hashed_table{$key}, $aref);
+		$hashed_table_overflow{$key} = \@new_overflow;
+		$hashed_table{$key} = 1;   # mark new overflow
+		$overflow_count++;
+	    };
+        } else {
+	    $hashed_table{$key} = $aref;
+	};
+    };
+
+    my %hashed_table_unmatched;
+    foreach (keys %hashed_table) {
+	$hashed_table_unmatched{$_} = 1;
+    };
+    my $out_fref = [];
+
+    my $build_output_from_full;
+    my $build_output_from_hit;
+    my $merge_output_from_hit;
+    my $empty = $self->{_empty};
+    my $hit;
+    my $reset_output_code = '$out_fref =  [ ($empty) x ' . $self->{_out}->ncols . " ];\n";
+    $code = '$build_output_from_full = sub {' . "\n" . $reset_output_code . $self->{_build_output_from_full_code} . "\n};\n" .
+	    '$build_output_from_hit = sub {' . "\n" . $reset_output_code . $self->{_build_output_from_hit_code} . "\n};\n" .
+	    '$merge_output_from_hit = sub {' . "\n" . $self->{_merge_output_from_hit_code} . "\n};\n";
+    eval $code;
+    $@ && croak $self->{_prog} . ":  internal eval error in hash probe code: $@.\n$code\n";
+
+    #
+    # Probe.
+    #
+    my($show_probe_non_matches) = ($self->{_join_type} eq 'outer') ||
+	($self->{_join_type} eq 'left' && $self->{_join_method} eq 'righthash') ||
+	($self->{_join_type} eq 'right' && $self->{_join_method} eq 'lefthash');
+    while ($fref = &{$fastpath_subs[$fullside]}()) {
+	# probe and join
+	my $key = &{$build_key_from_fullside}();
+        $hit = $hashed_table{$key};
+	if (!defined($hit)) {
+	    # no match
+	    if ($show_probe_non_matches) {
+		&{$build_output_from_full}();
+		&{$out_fastpath_sub}($out_fref);
+	    };
+	} else {
+	    delete $hashed_table_unmatched{$key};
+	    &{$build_output_from_full}();
+	    if (ref($hit) eq 'ARRAY') {
+		# single hit
+		&{$merge_output_from_hit}();
+		&{$out_fastpath_sub}($out_fref);
+	    } else {
+		# overflow hits
+		# xxx: if you replace "my $h" with "$hit", perl-5.22.2-362.fc24.x86_64 segfaults
+		foreach my $h (@{$hashed_table_overflow{$key}}) {
+		    $hit = $h;
+		    &{$merge_output_from_hit}();
+		    &{$out_fastpath_sub}($out_fref);
+		};
+	    };
+	};
+    };
+
+    #
+    # Dump extra hashs, if necessary.
+    #
+    if (($self->{_join_type} eq 'outer') ||
+		($self->{_join_type} eq 'left' && $self->{_join_method} eq 'lefthash') ||
+		($self->{_join_type} eq 'right' && $self->{_join_method} eq 'righthash')) {
+	for my $key (sort keys %hashed_table_unmatched) {
+	    $hit = $hashed_table{$key};
+	    if (ref($hit) eq 'ARRAY') {
+		# single hit
+		&{$build_output_from_hit}();
+		&{$out_fastpath_sub}($out_fref);
+	    } else {
+		# overflow hits
+		foreach $hit (@{$hashed_table_overflow{$key}}) {
+		    &{$build_output_from_hit}();
+		    &{$out_fastpath_sub}($out_fref);
+		};
+	    };
+	};	
+    };	
+}
+
+
+=head2 run
+
+    $filter->run();
+
+Internal: run over each rows.
+
+=cut
+sub run($) {
+    my($self) = @_;
+    if ($self->{_join_method} eq 'merge') {
+	$self->run_merge_join();
+    } elsif ($self->{_join_method} eq 'righthash') {
+	$self->run_hash_join(1, 0);
+    } elsif ($self->{_join_method} eq 'lefthash') {
+	$self->run_hash_join(0, 1);
+    } else {
+	die $self->{_prog} . ": unknown join method " . $self->{_join_method} . "\n";
+    };
+}
+
 =head1 AUTHOR and COPYRIGHT
 
-Copyright (C) 1991-2008 by John Heidemann <johnh@isi.edu>
+Copyright (C) 1991-2016 by John Heidemann <johnh@isi.edu>
 
 This program is distributed under terms of the GNU general
 public license, version 2.  See the file COPYING
