@@ -2,8 +2,7 @@
 
 #
 # dbmapreduce.pm
-# Copyright (C) 1991-2015 by John Heidemann <johnh@isi.edu>
-# $Id: 1e25691a19e98de45f517c2a454ccd40580c1348 $
+# Copyright (C) 1991-2016 by John Heidemann <johnh@isi.edu>
 #
 # This program is distributed under terms of the GNU general
 # public license, version 2.  See the file COPYING
@@ -276,6 +275,11 @@ but Fsdb::Filter objects in Perl do not run until you invoke
 the run() method.
 The C<--(no)autorun> option controls that behavior within Perl.
 
+=item B<--header> H
+
+Use H as the full Fsdb header, rather than reading a header from
+then input.
+
 =item B<--help>
 
 Show help.
@@ -432,6 +436,7 @@ sub set_defaults ($) {
     $self->{_max_parallelism} = undef;
     $self->{_parallelism_available} = undef;
     $self->{_test_parallelism} = undef;
+    $self->{_header} = undef;
     $self->set_default_tmpdir;
 }
 
@@ -457,6 +462,7 @@ sub parse_options ($@) {
 	'copy-fs|copy-fieldseparator!' => \$self->{_copy_fscode},
 	'd|debug+' => \$self->{_debug},
 	'f|code-files=s@' => $self->{_code_files},
+	'header=s' => \$self->{_header},
 	'i|input=s' => sub { $self->parse_io_option('input', @_); },
 	'k|key=s' => \$self->{_key_column},
 	'K|pass-current-key!' => \$self->{_pass_current_key},
@@ -562,22 +568,28 @@ sub setup($) {
     #
     # do we need to group the keys for the user?
     #
-    my($input_reader_aref) = [-comment_handler => $self->create_tolerant_pass_comments_sub('_cat_writer')];
+    my($input_reader_aref) = ();
     my $raw_to_raw = ($#{$self->{_external_command_argv}} >= 0 && $self->{_reducer_is_multikey_aware});
     if ($raw_to_raw) {
 	# external and we're good?  just hook it together
 	# (test case: dbmapreduce_cat.cmd)
 	$input_reader_aref = [-raw_fh => 1];
+    } else {
+	push (@$input_reader_aref, -comment_handler => $self->create_tolerant_pass_comments_sub('_cat_writer'));
     };
+    push(@$input_reader_aref, -header => $self->{_header}) if (defined($self->{_header}));
     if ($self->{_pre_sorted}) {
 	$self->finish_io_option('input', @$input_reader_aref);
     } else {
 	# not pre-sorted, so do lexical sort
 	my $sort_column = defined($self->{_key_column}) ? $self->{_key_column} : '0';
+	my(@sort_args) = ('--nolog');
+	push(@sort_args, '--header' => $self->{_header}) if (defined($self->{_header}));
+	push(@sort_args, $sort_column);
         my($new_reader, $new_fred) = dbpipeline_filter(
 		$self->{_input},
 		$input_reader_aref,
-		dbsort('--nolog', $sort_column));
+		dbsort(@sort_args));
 	$self->{_pre_sorted_input} = $self->{_input};
 	$self->{_in} = $new_reader;
 	$self->{_sorter_fred} = $new_fred;
@@ -938,7 +950,7 @@ sub finish($) {
 
 =head1 AUTHOR and COPYRIGHT
 
-Copyright (C) 1991-2015 by John Heidemann <johnh@isi.edu>
+Copyright (C) 1991-2016 by John Heidemann <johnh@isi.edu>
 
 This program is distributed under terms of the GNU general
 public license, version 2.  See the file COPYING
