@@ -3,7 +3,7 @@
 #
 # dbmultistats.pm
 # Copyright (C) 1991-2015 by John Heidemann <johnh@isi.edu>
-# $Id$
+# $Id: 85a9faaa887a82737100dceee7013e2894b800e1 $
 #
 # This program is distributed under terms of the GNU general
 # public license, version 2.  See the file COPYING
@@ -233,6 +233,7 @@ sub set_defaults ($) {
     $self->{_median} = undef;   # special case: renames the output field
     $self->{_max_parallelism} = undef;
     $self->{_include_non_numeric} = undef;
+    $self->{_header} = undef;
 }
 
 =head2 parse_options
@@ -257,6 +258,7 @@ sub parse_options ($@) {
 	'c|confidence=f' => \$self->{_confidence_fraction},
 	'd|debug+' => \$self->{_debug},
 	'f|format=s' => \$self->{_format},
+	'header=s' => \$self->{_header},
 	'i|input=s' => sub { $self->parse_io_option('input', @_); },
 	'k|key=s' => \$self->{_key_column},
 	'log!' => \$self->{_logprog},
@@ -312,6 +314,8 @@ sub setup ($) {
 #    push(@dbmapreduce_argv, qw(--noclose --saveoutput), \$self->{_out});
 #    $self->{_child_saves_output} = 1;
     # pass input and output
+    push (@dbmapreduce_argv, "--header", $self->{_header})
+	if (defined($self->{_header}));
     push (@dbmapreduce_argv, "--input", $self->{_input});
     push (@dbmapreduce_argv, "--output", $self->{_output});
     # the rest
