@@ -2,8 +2,7 @@
 
 #
 # dbsort.pm
-# Copyright (C) 1991-2015 by John Heidemann <johnh@isi.edu>
-# $Id$
+# Copyright (C) 1991-2016 by John Heidemann <johnh@isi.edu>
 #
 # This program is distributed under terms of the GNU general
 # public license, version 2.  See the file COPYING
@@ -111,6 +110,11 @@ but Fsdb::Filter objects in Perl do not run until you invoke
 the run() method.
 The C<--(no)autorun> option controls that behavior within Perl.
 
+=item B<--header> H
+
+Use H as the full Fsdb header, rather than reading a header from
+then input.
+
 =item B<--help>
 
 Show help.
@@ -210,6 +214,7 @@ sub set_defaults ($) {
     $self->{_sort_argv} = [];
     $self->set_default_tmpdir;
     $self->{_max_parallelism} = undef;
+    $self->{_header} = undef;
 }
 
 =head2 parse_options
@@ -231,6 +236,7 @@ sub parse_options ($@) {
 	'autorun!' => \$self->{_autorun},
 	'close!' => \$self->{_close},
 	'd|debug+' => \$self->{_debug},
+	'header=s' => \$self->{_header},
 	'i|input=s' => sub { $self->parse_io_option('input', @_); },
 	'log!' => \$self->{_logprog},
 	'M|maxmemory=i' => \$self->{_max_memory},
@@ -266,7 +272,9 @@ sub setup($) {
     #
     # setup final IO
     #
-    $self->finish_io_option('input', -comment_handler => $self->create_delay_comments_sub);
+    my(@finish_args) = (-comment_handler => $self->create_delay_comments_sub);
+    push (@finish_args, -header => $self->{_header}) if (defined($self->{_header}));
+    $self->finish_io_option('input', @finish_args);
 
     $self->{_compare_code} = $self->create_compare_code($self->{_in}, $self->{_in});
     croak $self->{_prog} . ": no sort field specified.\n"
